package com.faraphel.tasks_valider.connectivity.task.session

import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD
import java.util.*


/**
 * The manager for the session system
 */
class TaskSessionManager {
    private val sessions = mutableMapOf<String, TaskSession>()  ///< sessions specific data

    /**
     * Create a new session
     * @param session the data for the session (optional)
     * @param sessionId the session id to use (optional)
     * @return a new session identifier
     */
    fun newSessionData(
        session: TaskSession = TaskSession(),
        sessionId: String = UUID.randomUUID().toString()
    ): String {
        this.sessions[sessionId] = session
        return sessionId
    }

    /**
     * Get data from a http session
     * @param httpSession the HTTP session
     * @return the session data
     */
    fun getSessionData(httpSession: NanoHTTPD.IHTTPSession): TaskSession? {
        val sessionId = httpSession.cookies.read("sessionId") ?: return null
        val sessionData = this.getSessionData(sessionId)
        return sessionData
    }

    /**
     * Get data from a session identifier
     * @param sessionId the identifier of the session
     * @return the session data
     */
    fun getSessionData(sessionId: String): TaskSession? {
        return this.sessions[sessionId]
    }

    /**
     * Set the data of a session
     * @param sessionId the identifier of the session
     * @param session the session data
     */
    fun setSessionData(sessionId: String, session: TaskSession) {
        this.sessions[sessionId] = session
    }

    /**
     * Delete a session
     * @param sessionId the identifier of the session
     */
    fun deleteSessionData(sessionId: String): TaskSession? {
        return this.sessions.remove(sessionId)
    }

    /**
     * Get data from a http session. If it does not exist, create it.
     * @param httpSession the HTTP session
     */
    fun getOrCreateSessionData(httpSession: NanoHTTPD.IHTTPSession): TaskSession {
        // try to get the session directly
        var session = this.getSessionData(httpSession)

        // if the session does not exist, create it
        if (session == null) {
            val sessionId = this.newSessionData()
            session = this.getSessionData(sessionId)!!
        }

        // return the session
        return session
    }

    /**
     * Insert new cookies for a session in a response
     * @param response the response to inject cookies into
     * @param cookies the cookie handler
     */
    fun responseSetSessionData(
        response: NanoHTTPD.Response,
        cookies: NanoHTTPD.CookieHandler
    ): NanoHTTPD.Response {
        // update the cookie of the user
        cookies.set(NanoHTTPD.Cookie("sessionId", this.newSessionData()))
        // load them in the response
        cookies.unloadQueue(response)

        return response
    }
}